==> kite-1.0a4.tar.gz: kite-1.0a4/modules/System/vm/loader.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <string.h>
#include "kite_object.h"
#include "vm/kite_vm.h"

#define TYPECHECK_RIGHT_OPERAND(t) \
    kite_object_t *rhs = args->builtin_data.listvalue.car; \
    kite_object_t *exc; \
    if (rhs->type != t) { \
        exc = kite_new_exception(thd, "System.exceptions.TypeMismatch", \
                            "Parameter is not of the requested type."); \
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE); \
    }

#define METHOD_ADD(name, param) \
    kite_add_method(thread, newclass, # name, \
        kite_new_method(thread, FUNC_COMPILED, Loader_ ## name, param));

extern char *kite_dylib_path;
KITE_CLASS_METHOD(Loader_loadClass) {
    kite_symtab_t *found;
    TYPECHECK_RIGHT_OPERAND(OBJ_STRING);
    
    KITE_FIND_PROPERTY(found, this, "searchPath", 1);
    kite_dylib_path = found->value->builtin_data.stringvalue.string;
    kite_vm_return(thd, 
        kite_dereference_and_load(thd, rhs->builtin_data.stringvalue.string));
}

KITE_MODULE_INITIALIZER {
    /* Create new class object. inherits is typically NULL for packages. */
    kite_object_t *newclass = kite_new_class(thread, NULL, "loader");
    
    /* Add methods */
    METHOD_ADD(loadClass, 1);
    
    /* Add properties */
    kite_add_property(thread, newclass, "searchPath", 1);
    kite_set_property(thread, newclass, "searchPath", 
        kite_new_string(thread, kite_dylib_path));
    
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "loader", 1);
    kite_set_property(thread, parent, "loader", newclass);
}

==> kite_1.0.2.orig.tar.gz: kite-1.0.2/modules/System/vm/loader.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <string.h>
#include "kite_object.h"
#include "vm/kite_vm.h"

#define TYPECHECK_RIGHT_OPERAND(t) \
    kite_object_t *rhs = args->builtin_data.listvalue.car; \
    kite_object_t *exc; \
    if (rhs->type != t) { \
        exc = kite_new_exception(thd, "System.exceptions.TypeMismatch", \
                            "Parameter is not of the requested type."); \
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE); \
        return; \
    }

#define METHOD_ADD(name, param, doc, param1, desc1) \
    kite_add_method(thread, newclass, # name, \
        kite_new_method_compiled_with_docs(thread, Loader_ ## name, doc, param, param1, desc1));

extern char *kite_dylib_path;
KITE_CLASS_METHOD(Loader_loadClass) {
    kite_symtab_t *found;
    kite_object_t *compiled;

    TYPECHECK_RIGHT_OPERAND(OBJ_STRING);
    
    KITE_FIND_PROPERTY(found, this, "searchPath", 1);
    kite_dylib_path = found->value->builtin_data.stringvalue.string;
    compiled = kite_dereference_and_load(thd, rhs->builtin_data.stringvalue.string);
    if (compiled) kite_vm_return(thd, compiled);
}

KITE_MODULE_INITIALIZER(loader) {
    /* Create new class object. inherits is typically NULL for packages. */
    kite_object_t *newclass = kite_new_class(thread, NULL, "loader");
    kite_set_docstring(newclass, "User-level access to the Kite loader.");
    
    /* Add methods */
    METHOD_ADD(loadClass, 1, "Locates and loads class into memory.", 
               "name", "Name of class to load.");
    
    /* Add properties */
    kite_add_property(thread, newclass, "searchPath", 1, "Search path to Kite packages, separated by ':'.");
    kite_set_property(thread, newclass, "searchPath", 
        kite_new_string(thread, kite_dylib_path));
    
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "loader", 1, NULL);
    kite_set_property(thread, parent, "loader", newclass);
}
